#![recursion_limit = "256"]

use request::{ CustomParamRequest, CustomRequest, DoFastApi, UpdateFastApi };

use log::info;
use std::sync::Arc;
use structopt::StructOpt;
use tower_lsp::{LspService, Server};

mod core;
mod completion;
mod definition;
mod hover;
mod document_symbol;
mod document_highlight;
mod inlay_hint;
mod utils;
mod diagnostics;
mod format;
mod server;
mod sources;
mod request;

use server::Backend;

#[derive(StructOpt, Debug)]
#[structopt(name = "Digtal LSP", about = "LSP for Digital IDE")]
struct Opt {}

#[tokio::main]
async fn main() {
    let _ = Opt::from_args();
    let log_handle = flexi_logger::Logger::with(flexi_logger::LogSpecification::info())
        .start()
        .unwrap();
    
    info!("launch Digital LSP, version: 0.4.0");

    let stdin = tokio::io::stdin();
    let stdout = tokio::io::stdout();

    let (service, socket) = LspService::build(|client| {
        let backend = Arc::new(Backend::new(client, log_handle));
        backend
    })
        .custom_method("custom/request", CustomRequest)
        .custom_method("custom/paramRequest", CustomParamRequest)
        .custom_method("api/fast", DoFastApi)
        .custom_method("api/update-fast", UpdateFastApi)
        .finish();

    Server::new(stdin, stdout, socket)
        .serve(service)
        .await;
}
